//////////////////////////////////////////////////////////////////////
// scene.cpp: implementation of the scene class.
//
//////////////////////////////////////////////////////////////////////

#include "scene.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

scene::scene()
{
	int i;
	//////////////////////////////////////
	// smerniky na instancie na triedy nastavim na NULL, aby som ich potom nemazal
	font0 = NULL;
	//////////////////////////////////////
	// textury
	for(i=0;i<pocet_textur;i++)tex[i]=0;
	//////////////////////////////////////
	// prepinace
	svetlo=1; texturovanie=1; ciary=0;

	fps = 200;							// v pripade ze nieco zavisy od fps

	//////////////////////////////////////
	// nacitanie textur
	i=0;
	if( !LoadGLTextures("data/earth.bmp",		&tex[i++]) ) {error = 1;return;}

	//////////////////////////////////////
	// vytvarame instancie pouzivanych tried
	font0 = new Font("data/font.bmp",&error);		// vytvorenie fontu
	if(error)return;

	InitGL();									// nastavenie z-buffra,svetla,...
	//ShowCursor(0);
}

scene::~scene()
{
	//////////////////////////////////////
	// mazeme instancie tried
	if(font0!=NULL)delete font0;
	//////////////////////////////////////
	// mazeme textury
	for(int i=0;i<pocet_textur;i++)glDeleteTextures( 1, &tex[i]);
	KillGL();
	ShowCursor(1);
}

void scene::InitGL(void)
{
	if(!fullscreen)
	{
		RECT		WindowRect;					// miesto pre velkost okna
		WindowRect.left=(long)0;
		WindowRect.top=(long)0;
		GetClientRect(hWnd,&WindowRect);				//zistenie rozmerov uzivatelskej casti okna
		ReSizeGLScene(WindowRect.right,WindowRect.bottom);	//nastavenie perspektivy
	}
	else ReSizeGLScene(screen_x, screen_y);	//definovana v tomto subore

	glEnable(GL_TEXTURE_2D);		// Povolenie zobrazovanie textr
	glShadeModel(GL_SMOOTH);		/*nastavuje tienovanie
	GL_SMOOTH - objekty su tienovane t.j. prechody medzi farbami bodov su plinule (gouradovo (s)tienovanie)
	GL_FLAT   - objekty nie su tienovane ziadne plinule prechody */

	glClearColor(0.0f, 0.0f, 0.0f, 0.0f); //specifikuje hodnoty na cistenie pomocou glClear par.: red, green, blue, alpha (cierny podklad)
	glClearDepth(1.0f);  //specifikuje hodnoty na cistenie pomocou glClear par.: depth (hlbka)
	
	/* ZAPNUTIE HLBKOVEHO TESTU */
	glEnable(GL_DEPTH_TEST);		// zapne hbkov test
	glDepthFunc(GL_LEQUAL);			// typ hbkovho testu
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);	// relnejie perspektvne vpoty
	
	GLfloat LightAmbient[]	=	{ 0.2f, 0.2f, 0.2f, 1.0f };		//svetlo ktore osvetluje zo vsetkych stran rovnako (okolite svetlo)
	GLfloat LightDiffuse[]	=	{ 1.0f, 1.0f, 1.0f, 1.0f };		//smerove, alebo bodove svetlo - svetlo svieti z pozicie zadanej pomocou premennej LightPosition
	GLfloat LightPosition[4]=	{ 0.0f, 0.0f, 10.0f, 1.0f };		//smer difuzneho svetla - ak je 4 zlozka 0 - smerove svetlo, ak nie je 0 potom s jedna o bodove svetlo so suradnicami x,y,z
	
	glLightModelfv ( GL_LIGHT_MODEL_AMBIENT, LightAmbient );	// nastavenie okoliteho (Ambient) svetla
	glLightfv ( GL_LIGHT0, GL_DIFFUSE, LightDiffuse );			// nastavenie difuzneho svetla
	glLightfv ( GL_LIGHT0, GL_POSITION, LightPosition  );		// pozicia difuzneho svetla
	glEnable  ( GL_LIGHT0 );									// aktivovanie svetla 0
	
	GLfloat mat_diffuse[]	= { 1.0f, 1.0f, 1.0f, 1.0f };
	GLfloat mat_specular[]	= { 0.9f, 0.9f, 0.9f, 1.0f };
	GLfloat mat_shininess[] = { 128.0f };

	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE , mat_diffuse);
	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);

	glColor3f(1.0f,1.0f,1.0f);				// biela farba	
}

void scene::KillGL()
{
}


AUX_RGBImageRec* scene::LoadBMP(char *Filename)			// Natanie bitmapovho obrzka
{
	FILE *File=NULL;								// smernik na subor

	if (!Filename)return NULL;						// skka i meno sboru bolo zadan
	File=fopen(Filename,"r");						// otvorenie sboru na tanie
	if (File)										// existuje sbor ?
	{
		fclose(File);								// Uzatvorenie sbora
		return auxDIBImageLoad(Filename);			// Nata obrzok a vrti pointer
	}
	return NULL;									// Ak sa obrzok nenatal vrti NULL
}

int scene::LoadGLTextures(char *texture_name,GLuint *texture)				// Nata obrzok a uloi ako textury
{
	int Status=TRUE;								// indikator
	AUX_RGBImageRec *TextureImage;					// miesto pre truktru

	TextureImage=NULL;
	if (TextureImage=LoadBMP(texture_name))
	{
		glGenTextures(1, texture);
		glBindTexture(GL_TEXTURE_2D, *texture);
	//	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	//	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	//	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP );		// nastavuje za textury sa v smere u (vodorovnom) neopakuju
	//	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP );		// nastavuje za textury sa v smere v (zvislom) neopakuju
	//	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB8/*3*/, TextureImage->sizeX, TextureImage->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, TextureImage->data);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
		gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGB8/*3*/, TextureImage->sizeX, TextureImage->sizeY, GL_RGB, GL_UNSIGNED_BYTE, TextureImage->data);
	}
	else 
	{	MessageBox(NULL,"Chyba natania obrzka.",texture_name,MB_ICONERROR);Status=false;}

	if (TextureImage)									// Ak textura existuje
	{
		if (TextureImage->data)free(TextureImage->data);// ak existuju data uvoni z pamte
		free(TextureImage);								// uvoni truktru
	}
	return Status;
}

void scene::Kurzor(void)			// zisti poziciu mysi a zobrazy kurzor
{
/*	GetCursorPos(&MouseBod);		//nacitanie pozicie mysi
	glBegin(GL_LINES);
		glVertex2i(MouseBod.x+velkost_kurzoru+1,screen_y-1-MouseBod.y);
		glVertex2i(MouseBod.x-velkost_kurzoru+1,screen_y-1-MouseBod.y);
		glVertex2i(MouseBod.x+1,screen_y-1-MouseBod.y+velkost_kurzoru);
		glVertex2i(MouseBod.x+1,screen_y-1-MouseBod.y-velkost_kurzoru);
	glEnd();
*/
	glBegin(GL_LINES);
		glVertex2i(mouse_x+velkost_kurzoru+1,mouse_y);
		glVertex2i(mouse_x-velkost_kurzoru+1,mouse_y);
		glVertex2i(mouse_x+1,mouse_y+velkost_kurzoru);
		glVertex2i(mouse_x+1,mouse_y-velkost_kurzoru);
	glEnd();
}

void scene::ReSizeGLScene(int width, int height)
{
	sirka = width;
	vyska = height;
	ReSizeGLScene_2D(width, height);
}

void scene::ReSizeGLScene_3D(int width, int height)		// funkcia pre zmenu velkosti okna
{										// vola sa v pripade zmeny velkosti okna, zmenu z(na) fullscreen
	if (height==0) height=1;
	glViewport(0,0,width,height);		// Viewport (pohlad)
	glMatrixMode(GL_PROJECTION);		// projekcia
	glLoadIdentity();					// Reset pohladu - nacita jednotkovu maticu
	gluPerspective(uhol_kamery,(GLfloat)width/(GLfloat)height,blizka_orezavacia_rovina,vzdialena_orezavacia_rovina);
	/* gluPerspective - nastavuje perspektivu kamery, ako bude vypadat priestor z pohladu kamery.
	1. parameter urcuje zorn uhol kamery. 2.param. - gldAspect je pomer vky a rky zobrazovanej plochy.
	3. par. udava vzdialenost od ktorej je vidiet vykreslovane objekty
	4. par. udava vzdialenost do ktorej je vidiet vykreslovane objekty */
	glMatrixMode(GL_MODELVIEW);			// Modelview
	glLoadIdentity();					// Reset The Modelview Matrix
}

void scene::ReSizeGLScene_2D(int width, int height)		// funkcia pre zmenu velkosti okna
{
	if (height==0) height=1;
	glViewport(0,0,width,height);		// Viewport (pohlad)
	glMatrixMode(GL_PROJECTION);		// projekcia
	glLoadIdentity();					// Reset pohladu - nacita jednotkovu maticu
	glOrtho(0,screen_x,0,screen_y,-1,1);	// nastavuje standardne pravouhle premietanie
	glMatrixMode(GL_MODELVIEW);			// Modelview
	glLoadIdentity();					// Reset The Modelview Matrix
}

void scene::Prepni_do_3D(void)			// rychle prepnutie do 3D
{
	glEnable(GL_DEPTH_TEST );

	glMatrixMode(GL_PROJECTION);		// projekcia
	glLoadIdentity();					// Reset pohladu - nacita jednotkovu maticu
	gluPerspective(uhol_kamery,(float)sirka/(float)vyska,blizka_orezavacia_rovina,vzdialena_orezavacia_rovina);
	/* gluPerspective - nastavuje perspektivu kamery, ako bude vypadat priestor z pohladu kamery.
	1. parameter urcuje zorn uhol kamery. 2.param. - gldAspect je pomer vky a rky zobrazovanej plochy.
	3. par. udava vzdialenost od ktorej je vidiet vykreslovane objekty
	4. par. udava vzdialenost do ktorej je vidiet vykreslovane objekty */
	glMatrixMode(GL_MODELVIEW);			// Modelview
	glLoadIdentity();
}

void scene::Prepni_do_2D(void)			// rychle prepnutie do 2D
{
	if(svetlo)glDisable(GL_LIGHTING);					// vypne svetlo
	if(ciary)glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);	// vypne ciarovy rezim
	if(!texturovanie)glEnable(GL_TEXTURE_2D);			// zapne texturovanie
	glDisable(GL_DEPTH_TEST );

	glMatrixMode(GL_PROJECTION);		// projekcia
	glLoadIdentity();					// Reset pohladu - nacita jednotkovu maticu
	glOrtho(0,screen_x,0,screen_y,-1,1);// nastavuje standardne pravouhle premietanie
	glMatrixMode(GL_MODELVIEW);			// Modelview
	glLoadIdentity();					// Reset The Modelview Matrix
}
